import os
import re
from pathlib import Path


def main():
    tag = os.getenv('GITHUB_REF_NAME')
    if not tag:
        raise RuntimeError('tag not exists')
    if not tag.startswith('v'):
        raise ValueError(f'tag({tag}) not starts with "v"')
    print(f'tag: {tag}')
    init = Path('src/<package>/__init__.py')
    init.write_text(re.sub(
        r'^__version__ = [\'"]\d+\.\d+\.\d+[\'"]$',
        f"__version__ = '{tag[1:]}'",
        init.read_text(encoding='utf8'), flags=re.M
    ), encoding='utf8')


if __name__ == '__main__':
    main()



import os.path
from pathlib import Path
from textwrap import dedent

from pger import const


class PyPI:

    def __init__(self, user, repo, package, output=None):
        self.user = user
        self.repo = repo
        self.package = package
        self.output = Path(os.path.abspath(output) if output else repo)
        self.output.mkdir(parents=True, exist_ok=True)
        self.src = self.output / 'src'
        (self.src / package / const.TEMPLATES).mkdir(parents=True, exist_ok=True)
        self.tests = self.src / 'tests'
        self.workflows = self.output / '.github/workflows'
        self.workflows.mkdir(parents=True, exist_ok=True)
        self.pkg = self.src / package
        self.pkg.mkdir(parents=True, exist_ok=True)

    def manifest(self):
        (self.output / const.MANIFEST).write_text(
            (const.templates / const.MANIFEST).read_text().format(**self.__dict__)
        )

    def pyproject(self):
        (self.output / const.PYPROJECT).write_text(
            (const.templates / const.PYPROJECT).read_text().format(**self.__dict__)
        )

    def publish(self):
        (self.workflows / const.PUBLISH).write_text(
            (const.templates / const.PUBLISH).read_text()
        )

    def readme(self):
        (self.output / const.README).write_text(
            (const.templates / const.README).read_text().format(**self.__dict__)
        )

    def requirements(self):
        (self.output / const.REQUIREMENTS).write_text('')

    def started(self):
        (self.workflows / const.STARTED).write_text((const.templates / const.STARTED).read_text())

    def update(self):
        (self.workflows / const.UPDATE).write_text(
            (const.templates / const.UPDATE).read_text().replace('<package>', self.package)
        )

    def main(self):
        (self.pkg / const.MAIN).write_text(dedent(f"""
        from {self.package} import main

        if __name__ == '__main__':
            main()
        """).lstrip())

    def init(self):
        (self.pkg / const.INIT).write_text(dedent(f"""
        __title__ = '{self.repo}'
        __author__ = '{self.user}'
        __version__ = '0.0.1'

        import argparse


        def main():
            epilog = f'%(prog)s({{__version__}}) by {self.user}(https://github.com/{self.user}/{self.repo})'
            parser = argparse.ArgumentParser(prog='{self.repo}', description='', epilog=epilog)
            parser.add_argument('-v', '--version', action='version', version=epilog)
        
            parser.parse_args()
        """).lstrip())

    def gen_tests(self):
        self.tests.mkdir(parents=True, exist_ok=True)
        (self.tests / const.INIT).write_text('')
        (self.tests / const.CONFTEST).write_text(
            (const.templates / const.CONFTEST).read_text()
        )

    def generate(self):
        self.manifest()
        self.pyproject()
        self.publish()
        self.readme()
        self.requirements()
        self.started()
        self.update()
        self.main()
        self.init()
        self.gen_tests()
        return self.output



__title__ = 'pger'
__author__ = 'foyou'
__version__ = '0.0.12'

import argparse

from pger.pypi import PyPI


def main():
    epilog = f'%(prog)s({__version__}) by foyou(https://github.com/foyoux/pger)'
    parser = argparse.ArgumentParser(prog='pger', description='python package generator', epilog=epilog)
    parser.add_argument('-v', '--version', action='version', version=epilog)

    parser.add_argument('user', help='github username')
    parser.add_argument('-r', '--repo', help='github repository')
    parser.add_argument('-p', '--package', help='python package name -> src/<package>/__init__.py')

    parser.add_argument('-o', '--output', help='output directory')

    args = parser.parse_args()
    user = args.user
    if '/' in user:
        user, repo = user.split('/')
    else:
        repo = args.repo

    if not repo:
        print('repo not specify and first arg not contains repo')
        return

    package = args.package
    if package is None:
        if not repo[0].isalpha():
            print(f"package not specify and repo can\'t convert to package name, "
                  f"\nbecause repo doesn't start with the alpha")
            return
        else:
            package = repo.replace('-', '_')

    pkg = PyPI(user, repo, package, args.output)
    output = pkg.generate()

    print(f'🎉 Generate github repository ready at {output.absolute()}')



import os
from pathlib import Path

MANIFEST = 'MANIFEST.in'
PYPROJECT = 'pyproject.toml'
PUBLISH = 'python-publish.yml'
README = 'README.md'
REQUIREMENTS = 'requirements.txt'
STARTED = 'started-notify.yml'
UPDATE = 'update-version.py'
MAIN = '__main__.py'
INIT = '__init__.py'
TEMPLATES = 'templates'
CONFTEST = 'conftest.py'

templates = Path(os.path.join(os.path.dirname(__file__), TEMPLATES))
